[CmdletBinding()]
param
(
    [Parameter(Mandatory=$true)]
    [string] $subscriptionId,

    [Parameter(Mandatory=$true)]
    [string] $appServiceResourceGroup,

    [Parameter(Mandatory=$true)]
    [string] $appServiceName,

    [Parameter(Mandatory=$true)]
    [string] $storageAccountName,

    [Parameter(Mandatory=$true)]
    [string] $storageAccountResourceGroup,

    [string] $blobContainerName = 'nmm-backups',

    [int] $retentionPeriodInDays = 15
)

# log in and select subscription
Connect-AzAccount
Select-AzSubscription -SubscriptionId $subscriptionId
$user = Get-AzADUser -UserPrincipalName (Get-AzContext).Account.Id

# get app service
$appService = Get-AzWebApp -ResourceGroupName $appServiceResourceGroup -Name $appServiceName -ErrorAction Stop

# get app settings
$appSettings = @{}
ForEach ($setting in $appService.SiteConfig.AppSettings) {
	$appSettings[$setting.Name] = $setting.Value
}

$SQLServer = Get-AzSqlServer -ResourceGroupName $appServiceResourceGroup | Select-Object -ExpandProperty ServerName
$DBName = Get-AzSqlDatabase -ResourceGroupName $appServiceResourceGroup -ServerName $SQLServer | Where-Object { $_.DatabaseName -ne "Master"} | Select-Object -ExpandProperty DatabaseName

# ensure DB is not too large
$SqlDb = Get-AzSqlDatabase -ResourceGroupName $appServiceResourceGroup -ServerName $SQLServer -DatabaseName $DBName
$dbResource = get-azresource -ResourceId $SqlDb.ResourceId
$dbMetricStorage = $dbResource | Get-AzMetric -MetricName 'storage'
$dbUsedSpace = $dbMetricStorage.Data.Maximum | Select-Object -Last 1
$dbUsedSpace = [math]::Round($dbUsedSpace / 1GB, 2)


If ($dbUsedSpace -gt 4){
	Write-Error "App Service can only back up database to a max size of 4 GB. Due to the size of this database, native SQL backups will need to be used instead." -ErrorAction Stop
}

# check if DefaultConnection string is saved on app service or in key vault; if in key vault, grant permission and obtain string
if ($appService.SiteConfig.ConnectionStrings.ConnectionString) {
	$connectionString = $appService.SiteConfig.ConnectionStrings.ConnectionString
} else {
	# get keyVault name from app settings
	$vaultName = $appSettings["Deployment:KeyVaultName"]
	$vault = Get-AzKeyVault -VaultName $vaultName -ErrorAction Stop

	# ensure permission to get secrets
	$userPermissions = $vault.AccessPolicies | Where-Object { $_.ObjectId -eq $user.Id}
	if ($userPermissions.PermissionsToSecrets -notcontains "list" -or $userPermissions.PermissionsToSecrets -notcontains "get") {
		$permissionsToSecrets = New-Object Collections.Generic.List[string]
		if ($userPermissions) {
			$permissionsToSecrets.AddRange($userPermissions.PermissionsToSecrets)
		}
		if ($permissionsToSecrets -notcontains "get") {
			$permissionsToSecrets.Add("get")
		}
		if ($permissionsToSecrets -notcontains "list") {
			$permissionsToSecrets.Add("list")
		}
		Set-AzKeyVaultAccessPolicy -VaultName $vaultName -ObjectId $user.Id -PermissionsToSecrets $permissionsToSecrets
	}

	# get DB connection string from keyVault
	$defaultConnectionSecret = Get-AzKeyVaultSecret -VaultName $vaultName -Name "ConnectionStrings--DefaultConnection" -ErrorAction Stop -WarningAction Ignore
	$connectionString = [System.Net.NetworkCredential]::new("", $defaultConnectionSecret.SecretValue).Password
}

# create db settings object for backup configuration
$dbSettings = New-Object -TypeName Microsoft.Azure.Management.WebSites.Models.DatabaseBackupSetting
$dbSettings.Name = "$(($appServiceName -split "-")[0])-app-db" # 'nmw' or 'nwm' depending on age of deployment
$dbSettings.DatabaseType = "SqlAzure"
$dbSettings.ConnectionStringName = "DefaultConnection"
$dbSettings.ConnectionString = $connectionString

# configure storage account context
$keys = Get-AzStorageAccountKey -ResourceGroupName $storageAccountResourceGroup -Name $storageAccountName
$storageContext = New-AzStorageContext -StorageAccountName $storageAccountName -StorageAccountKey $keys[0].Value

# make sure blob container exists
$blobContainer = Get-AzStorageContainer -Context $storageContext -Name $blobContainerName -ErrorAction SilentlyContinue
if (!$blobContainer) {
    $blobContainer = New-AzStorageContainer -Context $storageContext -Name $blobContainerName -Permission Off
}

# generate sasUri for blob container
$sasUri = New-AzStorageContainerSASToken -Context $storageContext -Name $blobContainerName -Permission rwdl -ExpiryTime (Get-Date).AddYears(10) -FullUri

# enable backups
Edit-AzWebAppBackupConfiguration -WebApp $appService -FrequencyInterval 1 -FrequencyUnit Day -RetentionPeriodInDays $retentionPeriodInDays -KeepAtLeastOneBackup -StartTime (Get-Date) -StorageAccountUrl $sasUri -Databases $dbSettings -Enabled -ErrorAction Stop