﻿[CmdletBinding()]
param 
(
    [Parameter(Mandatory=$true)]
    [string] $subscription,

    [Parameter(Mandatory=$true)]
    [string] $vaultName
)

# prepare files structure
$targetFolder = Join-Path -Path $PSScriptRoot -ChildPath keyvault-backup
$secretsPath = Join-Path -Path $targetFolder secrets
$certificatesPath = Join-Path -Path $targetFolder certificates
$keysPath = Join-Path -Path $targetFolder keys

if (Test-Path $targetFolder) {
    Remove-Item -Path $targetFolder -Recurse -Force
}

New-Item -ItemType directory -Path $targetFolder -Name secrets
New-Item -ItemType directory -Path $targetFolder -Name certificates
New-Item -ItemType directory -Path $targetFolder -Name keys

# connect to azure
Connect-AzAccount
Select-AzSubscription -SubscriptionId $subscription

# get user running the script
$context = Get-AzContext
$user = Get-AzADUser -UserPrincipalName $context.Account.Id
#$user = Get-AzADApplication -ApplicationId $context.Account.Id

# ensure 'backup' and 'list' permissions for keyVault secrets and certificates
$vault = Get-AzKeyVault -VaultName $vaultName
$userPermissions = $vault.AccessPolicies | Where-Object { $_.ObjectId -eq $user.Id}
$permissionsToKeys = New-Object Collections.Generic.List[string]
$permissionsToSecrets = New-Object Collections.Generic.List[string]
$permissionsToCert = New-Object Collections.Generic.List[string]

if ($userPermissions) {
    $permissionsToKeys.AddRange($userPermissions.PermissionsToKeys)
    $permissionsToSecrets.AddRange($userPermissions.PermissionsToSecrets)
    $permissionsToCert.AddRange($userPermissions.PermissionsToCertificates)
}

if (!$permissionsToKeys.Contains("backup")) {
    $permissionsToKeys.Add("backup")
}

if (!$permissionsToKeys.Contains("list")) {
    $permissionsToKeys.Add("list")
}

if (!$permissionsToSecrets.Contains("backup")) {
    $permissionsToSecrets.Add("backup")
}

if (!$permissionsToSecrets.Contains("list")) {
    $permissionsToSecrets.Add("list")
}

if (!$permissionsToCert.Contains("backup")) {
    $permissionsToCert.Add("backup")
}

if (!$permissionsToCert.Contains("list")) {
    $permissionsToCert.Add("list")
}

Set-AzKeyVaultAccessPolicy -VaultName $vaultName -ObjectId $user.Id -PermissionsToKeys $permissionsToKeys -PermissionsToSecrets $permissionsToSecrets -PermissionsToCertificates $permissionsToCert

# load secrets/certificates/keys
$secrets = Get-AzKeyVaultSecret -VaultName $vaultName
$certificates = Get-AzKeyVaultCertificate -VaultName $vaultname
$keys = Get-AzKeyVaultKey -VaultName $vaultname

# removing duplicate keys
$keys = $keys | Where-Object { $_.Name -notin $certificates.Name }

# backup secrets
$secrets | Where-Object { $_.ContentType -ne 'application/x-pkcs12' } | ForEach-Object {
    Write-Host 'Processing secret: ' $_.Name
    $itemPath = Join-Path -Path $secretsPath -ChildPath $_.Name
    Backup-AzKeyVaultSecret -VaultName $vaultName -Name $_.Name -OutputFile $itemPath -Force
}

# backup certificates
$certificates | ForEach-Object {
    Write-Host 'Processing certificate: ' $_.Name
    $itemPath = Join-Path -Path $certificatesPath -ChildPath $_.Name
    Backup-AzKeyVaultCertificate -VaultName $vaultName -Name $_.Name -OutputFile $itemPath -Force
}

# backup keys
$Keys | ForEach-Object {
    Write-Host 'Processing keys: ' $_.Name
    $itemPath = Join-Path -Path $keysPath -ChildPath $_.Name
    Backup-AzKeyVaultKey -VaultName $vaultName -Name $_.Name -OutputFile $itemPath -Force
}

# zip files
$zipPath = Join-Path $PSScriptRoot keyvault-backup.zip
Compress-Archive -Path (Join-Path $targetFolder *) -DestinationPath $zipPath -Force
Remove-Item $targetFolder -Recurse -Force